@@ -2,14 +2,16 @@
 import crocodile.toolbox as tb
 import platform
 import os
+import sys
 
+P = tb.P
+L = tb.L
 
 system = platform.system()
 OS = os.getenv("OS")  # Windows_NT
 sep = ";" if system == "Windows" else ":"  # path separator, not to be confused with P.sep
-
-P = tb.P
-L = tb.List
+env = tb.Struct(dict(os.environ)).clean_view
+exe = P(sys.executable)
 
 tm = tb.Terminal()
 
@@ -18,7 +20,7 @@ DotFiles = P.home().joinpath("dotfiles")
 
 LocalAppData = P(tmp) if (tmp := os.getenv("LOCALAPPDATA")) else None  # C:\Users\username\AppData\Local
 AppData = P(tmp) if (tmp := os.getenv("APPDATA")) else None  # C:\Users\username\AppData\Roaming
-WindowsApps = AppData.joinpath(r"Microsoft\WindowsApps") if AppData else None  # this path is already in PATH. Thus, useful to add symlinks and shortcuts to apps that one would like to be in the PATH.
+WindowsApps = LocalAppData.joinpath(r"Microsoft\WindowsApps") if AppData else None  # this path is already in PATH. Thus, useful to add symlinks and shortcuts to apps that one would like to be in the PATH.
 
 ProgramData = P(tmp) if (tmp := os.getenv("PROGRAMDATA")) else None  # C:\ProgramData
 ProgramFiles = P(tmp) if (tmp := os.getenv("ProgramFiles")) else None  # C:\Program Files
@@ -27,8 +29,7 @@ ProgramFilesX86 = P(tmp) if (tmp := os.getenv("ProgramFiles(x86)")) else None  #
 
 CommonProgramFiles = P(tmp) if (tmp := os.getenv("CommonProgramFiles")) else None  # C:\Program Files\Common Files
 CommonProgramW6432 = P(tmp) if (tmp := os.getenv("CommonProgramW6432")) else None  # C:\Program Files\Common Files
-CommonProgramFilesX86 = P(tmp) if \
-    (tmp := os.getenv("CommonProgramFiles(x86)")) else None  # C:\Program Files (x86)\Common Files
+CommonProgramFilesX86 = P(tmp) if (tmp := os.getenv("CommonProgramFiles(x86)")) else None  # C:\Program Files (x86)\Common Files
 
 Tmp = P(tmp) if (tmp := os.getenv("TMP")) else None  # C:\Users\usernrame\AppData\Local\Temp
 Temp = Tmp
@@ -40,9 +41,9 @@ HostName = os.getenv("COMPUTERNAME")  # e.g. "MY-SURFACE"
 LogonServer = os.getenv("LOGONSERVER")  # e.g. "\\MY-SURFACE"
 UserDomain = os.getenv("USERDOMAIN")  # e.g. HAD OR MY-SURFACE
 UserDomainRoaming = P(tmp) if (tmp := os.getenv("USERDOMAIN_ROAMINGPROFILE")) else None  # e.g. SURFACE
-UserName = os.getenv("USERNAME")  # e.g: alex
-UserProfile = P(tmp) if (tmp := os.getenv("USERPROFILE")) else None  # e.g C:\Users\eng_a
-HomePath = P(tmp) if (tmp := os.getenv("HOMEPATH")) else None  # e.g. C:\Users\eng_a
+UserName = os.getenv("USERNAME")  # e.g: username
+UserProfile = P(tmp) if (tmp := os.getenv("USERPROFILE")) else None  # e.g C:\Users\username
+HomePath = P(tmp) if (tmp := os.getenv("HOMEPATH")) else None  # e.g. C:\Users\username
 Public = P(tmp) if (tmp := os.getenv("PUBLIC")) else None  # C:\Users\Public
 
 OneDriveConsumer = P(tmp) if (tmp := os.getenv("OneDriveConsumer")) else None
@@ -56,23 +57,18 @@ OneDriveExe = LocalAppData.joinpath("Microsoft/OneDrive/OneDrive.exe") if LocalA
 
 def get_address():
     netifaces = tb.install_n_import("netifaces")
-    subnet_mask = netifaces.ifaddresses(netifaces.gateways()['default'][netifaces.AF_INET][1])[netifaces.AF_INET][0][
-        'netmask']
+    subnet_mask = netifaces.ifaddresses(netifaces.gateways()['default'][netifaces.AF_INET][1])[netifaces.AF_INET][0]['netmask']
     default_gateway = netifaces.gateways()['default'][netifaces.AF_INET][0]
-
     import uuid
     mac = uuid.getnode()
     mac_address = ":".join(("%012X" % mac)[i:i + 2] for i in range(0, 12, 2))
     # elif hex_format: return hex(mac)
     # else: return mac
-
     import socket
     local_ip_v4 = socket.gethostbyname(socket.gethostname())
-
     from requests import get
     public_ip = get('https://api.ipify.org').text
-    return dict(subnet_mask=subnet_mask, mac_address=mac_address, local_ip_v4=local_ip_v4,
-                default_gateway=default_gateway, public_ip=public_ip)
+    return dict(subnet_mask=subnet_mask, mac_address=mac_address, local_ip_v4=local_ip_v4, default_gateway=default_gateway, public_ip=public_ip)
 
 
 # ============================== System Variables ==============================
@@ -84,7 +80,6 @@ class ShellVar(object):
         if system == "Windows":
             res = f"set {key} {val}"
             return res if not run else tm.run(res, shell=shell)
-
         elif system == "Linux":
             res = f"{key} = {val}"
             return res if not run else tm.run(res, shell="bash")
@@ -135,12 +130,9 @@ class PathVar:
     def append_temporarily(path, kind="append", run=False):
         if system == "Windows":
             """Source: https://docs.microsoft.com/en-us/powershell/module/microsoft.powershell.core/about/about_environment_variables?view=powershell-7.2"""
-            if kind == "append":  # Append to the Path variable in the current window:
-                command = fr'$env:Path += ";{path}"'
-            elif kind == "prefix":  # Prefix the Path variable in the current window:
-                command = fr'$env:Path = "{path};" + $env:Path'
-            elif kind == "replace":  # Replace the Path variable in the current window (use with caution!):
-                command = fr'$env:Path = "{path}"'
+            if kind == "append": command = fr'$env:Path += ";{path}"'  # Append to the Path variable in the current window:
+            elif kind == "prefix": command = fr'$env:Path = "{path};" + $env:Path'  # Prefix the Path variable in the current window:
+            elif kind == "replace": command = fr'$env:Path = "{path}"'  # Replace the Path variable in the current window (use with caution!):
             else: raise KeyError
             return command if run is False else tm.run(command, shell="powershell")
         else: result = f'export PATH="{path}:$PATH"'
@@ -177,8 +169,6 @@ class PathVar:
         if system == "Windows":
             result = fr'[System.Environment]::GetEnvironmentVariable("Path","Machine") + ";" + [System.Environment]::GetEnvironmentVariable("Path","User")'
             return result if run is False else tm.run(result, shell="powershell")
-        else:
-            raise NotImplementedError
 
 
 # ============================== Shells =========================================
@@ -197,6 +187,7 @@ def get_shell_profiles(shell):
 
 
 def construct_path(path_list): return tb.L(__import__("pd").unique(path_list)).reduce(lambda x, y: str(x) + sep + str(y))
+def get_defined_programs(string_="*.exe"): return tb.P.env().Path.search(string_).reduce(lambda x, y: x+y).print()
 
 
 if __name__ == '__main__':
