@@ -16,7 +16,7 @@ from crocodile import file_management as _fm
 str2timedelta, timestamp, randstr, validate_name, install_n_import = core.str2timedelta, core.timestamp, core.randstr, core.validate_name, core.install_n_import
 Base, Struct, Display, Save, List = core.Base, core.Struct, core.Display, core.Save, core.List
 # File Management ==========================
-P, Read, Compression, Fridge, MemoryDB, encrypt, decrypt = _fm.P, _fm.Read, _fm.Compression, _fm.Fridge, _fm.MemoryDB, _fm.encrypt, _fm.decrypt
+P, Read, Compression, Fridge, encrypt, decrypt = _fm.P, _fm.Read, _fm.Compression, _fm.Fridge, _fm.encrypt, _fm.decrypt
 # META ====================================
 Experimental, Terminal, Log, Null, Scheduler, SSH = meta.Experimental, meta.Terminal, meta.Log, meta.Null, meta.Scheduler, meta.SSH
 
@@ -46,7 +46,6 @@ datetime = _fm.datetime
 #     _ = pd
 
 
-
 if __name__ == '__main__':
     # get_parser()
     pass
